@@ -58,10 +58,9 @@ def expandPrefix(prefix):
         return prefix + '.'
     return prefix
 
+@interface.implementer(interfaces.IFormField)
 class FormField:
 
-    interface.implements(interfaces.IFormField)
-
     def __init__(self, field, name=None, prefix='',
                  for_display=None, for_input=None, custom_widget=None,
                  render_context=False, get_rendered=None, interface=None
@@ -88,10 +87,9 @@ def _initkw(keep_readonly=(), omit_readonly=False, **defaults):
     return keep_readonly, omit_readonly, defaults
 
 
+@interface.implementer(interfaces.IFormFields)
 class FormFields(object):
 
-    interface.implements(interfaces.IFormFields)
-
     def __init__(self, *args, **kw):
         keep_readonly, omit_readonly, defaults = _initkw(**kw)
 
@@ -178,10 +176,9 @@ def fields(*args, **kw):
     return FormFields(*args, **other)
 
 
+@interface.implementer(interfaces.IWidgets)
 class Widgets(object):
 
-    interface.implements(interfaces.IWidgets)
-
     def __init__(self, widgets, prefix_length=None, prefix=None):
         self.__Widgets_widgets_items__ = widgets
         self.__Widgets_widgets_list__ = [w for (i, w) in widgets]
@@ -459,13 +456,15 @@ class NoInputData(interface.Invalid):
 
 class FormData:
 
-    def __init__(self, schema, data):
+    def __init__(self, schema, data, context):
         self._FormData_data___ = data
         self._FormData_schema___ = schema
+        self._FormData_context___ = context
 
     def __getattr__(self, name):
         schema = self._FormData_schema___
         data = self._FormData_data___
+        context = self._FormData_context___
         try:
             field = schema[name]
         except KeyError:
@@ -473,7 +472,21 @@ class FormData:
         else:
             value = data.get(name, data)
             if value is data:
-                raise NoInputData(name)
+                if context is None:
+                    raise NoInputData(name)
+                # The value is not in the form look it up on the context:
+                field = schema[name]
+                adapted_context = schema(context)
+                if IField.providedBy(field):
+                    value = field.get(adapted_context)
+                elif (zope.interface.interfaces.IAttribute.providedBy(field)
+                      and
+                      not zope.interface.interfaces.IMethod.providedBy(field)):
+                    # Fallback for non-field schema contents:
+                    value = getattr(adapted_context, name)
+                else:
+                    # Don't know how to extract value
+                    raise NoInputData(name)
             if zope.interface.interfaces.IMethod.providedBy(field):
                 if not IField.providedBy(field):
                     raise RuntimeError(
@@ -486,7 +499,7 @@ class FormData:
         raise AttributeError(name)
 
 
-def checkInvariants(form_fields, form_data):
+def checkInvariants(form_fields, form_data, context):
 
     # First, collect the data for the various schemas
     schema_data = {}
@@ -506,7 +519,7 @@ def checkInvariants(form_fields, form_data):
     errors = []
     for schema, data in schema_data.items():
         try:
-            schema.validateInvariants(FormData(schema, data), errors)
+            schema.validateInvariants(FormData(schema, data, context), errors)
         except interface.Invalid:
             pass # Just collect the errors
 
@@ -552,9 +565,8 @@ def _callify(meth):
         return lambda form, *args: getattr(form, meth)(*args)
 
 
+@interface.implementer(interfaces.IAction)
 class Action(object):
-
-    interface.implements(interfaces.IAction)
     _identifier = re.compile('[A-Za-z][a-zA-Z0-9_]*$')
 
     def __init__(self, label, success=None, failure=None,
@@ -660,10 +672,9 @@ class action:
         return action
 
 
+@interface.implementer(interfaces.IActions)
 class Actions(object):
 
-    interface.implements(interfaces.IActions)
-
     def __init__(self, *actions):
         self.actions = actions
         self.byname = dict([(a.__name__, a) for a in actions])
@@ -717,6 +728,7 @@ def availableActions(form, actions):
     return result
 
 
+@interface.implementer(interfaces.IForm)
 class FormBase(zope.publisher.browser.BrowserPage):
 
     label = u''
@@ -727,7 +739,8 @@ class FormBase(zope.publisher.browser.BrowserPage):
 
     errors = ()
 
-    interface.implements(interfaces.IForm)
+    ignoreContext = False
+
 
     def setPrefix(self, prefix):
         self.prefix = prefix
@@ -739,8 +752,12 @@ class FormBase(zope.publisher.browser.BrowserPage):
             form=self, adapters=self.adapters, ignore_request=ignore_request)
 
     def validate(self, action, data):
+        if self.ignoreContext:
+            context = None
+        else:
+            context = self.context
         return (getWidgetsData(self.widgets, self.prefix, data)
-                + checkInvariants(self.form_fields, data))
+                + checkInvariants(self.form_fields, data, context))
 
     template = namedtemplate.NamedTemplate('default')
 
@@ -793,7 +810,7 @@ class FormBase(zope.publisher.browser.BrowserPage):
 
     def __call__(self):
         self.update()
-        if self.request.response.getStatus() in [301, 302]:
+        if self.request.response.getStatus() in [301, 302, 303, 307]:
             # Avoid rendering if the action caused a redirect.
             result = self.form_result or ''
         else:
@@ -873,13 +890,14 @@ class DisplayFormBase(FormBase):
     actions = ()
 
 
+@interface.implementer(interfaces.IAddFormCustomization,
+                       zope.component.interfaces.IFactory)
+@component.adapter(zope.browser.interfaces.IAdding,
+                   zope.publisher.interfaces.browser.IBrowserRequest)
 class AddFormBase(FormBase):
 
-    interface.implements(interfaces.IAddFormCustomization,
-                         zope.component.interfaces.IFactory)
+    ignoreContext = True
 
-    component.adapts(zope.browser.interfaces.IAdding,
-                     zope.publisher.interfaces.browser.IBrowserRequest)
 
     def __init__(self, context, request):
         self.__parent__ = context
@@ -929,38 +947,38 @@ default_page_template = namedtemplate.NamedTemplateImplementation(
 default_subpage_template = namedtemplate.NamedTemplateImplementation(
     ViewPageTemplateFile('subpageform.pt'), interfaces.ISubPageForm)
 
+@interface.implementer(interfaces.IPageForm)
 class PageForm(FormBase):
-
-    interface.implements(interfaces.IPageForm)
+    pass
 
 Form = PageForm
 
+@interface.implementer(interfaces.IPageForm)
 class PageEditForm(EditFormBase):
-
-    interface.implements(interfaces.IPageForm)
+    pass
 
 EditForm = PageEditForm
 
+@interface.implementer(interfaces.IPageForm)
 class PageDisplayForm(DisplayFormBase):
-
-    interface.implements(interfaces.IPageForm)
+    pass
 
 DisplayForm = PageDisplayForm
 
+@interface.implementer(interfaces.IPageForm)
 class PageAddForm(AddFormBase):
-
-    interface.implements(interfaces.IPageForm)
+    pass
 
 AddForm = PageAddForm
 
+@interface.implementer(interfaces.ISubPageForm)
 class SubPageForm(FormBase):
+    pass
 
-    interface.implements(interfaces.ISubPageForm)
-
+@interface.implementer(interfaces.ISubPageForm)
 class SubPageEditForm(EditFormBase):
+    pass
 
-    interface.implements(interfaces.ISubPageForm)
-
+@interface.implementer(interfaces.ISubPageForm)
 class SubPageDisplayForm(DisplayFormBase):
-
-    interface.implements(interfaces.ISubPageForm)
+    pass
