@@ -17,7 +17,7 @@ __docformat__ = 'restructuredtext'
 from xml.sax.saxutils import escape
 
 from zope import component
-from zope.interface import implements
+from zope.interface import implementer
 from zope.i18n import translate
 from zope.schema.interfaces import InvalidValue
 from zope.schema.interfaces import ITitledTokenizedTerm
@@ -299,13 +299,13 @@ class SetDisplayWidget(ItemsMultiDisplayWidget):
 EXPLICIT_EMPTY_SELECTION = True
 
 
+@implementer(IInputWidget)
 class ItemsEditWidgetBase(SingleDataHelper, ItemsWidgetBase):
     """Widget Base for rendering item-related fields.
 
     These widgets work with Choice fields and Sequence fields that have Choice
     as value_type.
     """
-    implements(IInputWidget)
 
     size = 5
     tag = 'select'
