@@ -16,7 +16,7 @@
 __docformat__ = 'restructuredtext'
 
 from zope import component
-from zope.interface import implements
+from zope.interface import implementer
 from zope.i18n import translate
 from zope.schema.interfaces import ValidationError
 
@@ -29,6 +29,7 @@ from zope.formlib.widget import DisplayWidget, renderElement
 from zope.browserpage import ViewPageTemplateFile
 
 
+@implementer(IInputWidget)
 class SequenceWidget(BrowserWidget, InputWidget):
     """A widget baseclass for a sequence of fields.
 
@@ -36,8 +37,6 @@ class SequenceWidget(BrowserWidget, InputWidget):
                  items in the sequence
     """
 
-    implements(IInputWidget)
-
     template = ViewPageTemplateFile('sequencewidget.pt')
 
     _type = tuple
