@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -18,8 +18,8 @@ from itertools import imap
 
 import xml.sax.saxutils
 
-from zope.component import adapts, getMultiAdapter
-from zope.interface import implements
+from zope.component import adapter, getMultiAdapter
+from zope.interface import implementer
 import zope.browser.interfaces
 import zope.schema.interfaces
 from zope.schema.interfaces import \
@@ -43,10 +43,9 @@ from zope.formlib.widget import InputWidget
 import zope.formlib.itemswidgets
 from zope.formlib.widget import DisplayWidget
 
+@implementer(IDisplayWidget)
 class SourceDisplayWidget(DisplayWidget):
 
-    implements(IDisplayWidget)
-
     def __init__(self, field, source, request):
         super(SourceDisplayWidget, self).__init__(field, request)
         self.source = source
@@ -93,6 +92,8 @@ class SourceDisplayWidget(DisplayWidget):
 
 class SourceSequenceDisplayWidget(SourceDisplayWidget):
 
+    separator = '<br />\n'
+
     def __call__(self):
 
         if self._renderedValueSet():
@@ -114,14 +115,14 @@ class SourceSequenceDisplayWidget(SourceDisplayWidget):
 
             result.append(value)
 
-        return '<br />\n'.join(result)
+        return self.separator.join(result)
 
 
+@implementer(IInputWidget)
 class SourceInputWidget(InputWidget):
 
     _error = None
 
-    implements(IInputWidget)
 
     def __init__(self, field, source, request):
         super(SourceInputWidget, self).__init__(field, request)
@@ -511,6 +512,8 @@ class SourceListInputWidget(SourceInputWidget):
 # should be updated into full implementations.
 
 
+@implementer(IVocabularyTokenized)
+@adapter(IIterableSource)
 class IterableSourceVocabulary(object):
 
     """Adapts an iterable source into a legacy vocabulary.
@@ -520,9 +523,6 @@ class IterableSourceVocabulary(object):
     registered to obtain the terms.
     """
 
-    implements(IVocabularyTokenized)
-    adapts(IIterableSource);
-
     def __init__(self, source, request):
         self.source = source
         self.terms = getMultiAdapter((source, request),
