@@ -12,12 +12,10 @@
 #
 ##############################################################################
 """TextWidget Tests
-
-$Id: test_functional_textwidget.py 112015 2010-05-05 18:27:23Z tseaver $
 """
 import unittest
 
-from zope.interface import Interface, implements
+from zope.interface import Interface, implementer
 from zope.schema import TextLine, Choice
 from zope.formlib import form
 from zope.publisher.browser import TestRequest
@@ -42,10 +40,9 @@ class ITextLineTest(Interface):
         required=False,
         values=(u'Bob', u'is', u'Your', u'Uncle'))
 
+@implementer(ITextLineTest)
 class TextLineTest(object):
 
-    implements(ITextLineTest)
-
     def __init__(self):
         self.s1 = ''
         self.s2 = u'foo'
