@@ -18,7 +18,7 @@ __docformat__ = 'restructuredtext'
 from xml.sax.saxutils import quoteattr, escape
 
 from zope.component import getMultiAdapter, provideAdapter
-from zope.interface import implements
+from zope.interface import implementer
 from zope.schema.interfaces import ValidationError
 from zope.publisher.browser import BrowserView
 from zope.publisher.interfaces.browser import IDefaultBrowserLayer
@@ -44,11 +44,10 @@ if quoteattr("\r") != '"&13;"':
             data, {'\n': '&#10;', '\r': '&#13;', '\t':'&#9;'})
 
 
+@implementer(IWidget)
 class Widget(object):
     """Mixin class providing functionality common across widget types."""
 
-    implements(IWidget)
-
     _prefix = 'field.'
     _data_marker = object()
     _data = _data_marker
@@ -103,9 +102,9 @@ class InputWidget(Widget):
             return False
 
 
+@implementer(IWidgetFactory)
 class CustomWidgetFactory(object):
     """Custom Widget Factory."""
-    implements(IWidgetFactory)
 
     def __init__(self, widget_factory, *args, **kw):
         self._widget_factory = widget_factory
@@ -133,6 +132,7 @@ class CustomWidgetFactory(object):
 
         return self._create(args)
 
+@implementer(IBrowserWidget)
 class BrowserWidget(Widget, BrowserView):
     """Base class for browser widgets.
 
@@ -167,8 +167,8 @@ class BrowserWidget(Widget, BrowserView):
     illustrate, we can create and register a simple error display view:
 
         >>> from zope.formlib.interfaces import IWidgetInputError
-        >>> class SnippetErrorView:
-        ...     implements(IWidgetInputErrorView)
+        >>> @implementer(IWidgetInputErrorView)
+        ... class SnippetErrorView:
         ...     def __init__(self, context, request):
         ...         self.context = context
         ...     def snippet(self):
@@ -176,7 +176,7 @@ class BrowserWidget(Widget, BrowserView):
         >>> provideAdapter(SnippetErrorView,
         ...                (IWidgetInputError, IDefaultBrowserLayer),
         ...                IWidgetInputErrorView, '')
-        
+
     Whever an error occurs, widgets should set _error:
 
         >>> widget._error = WidgetInputError('foo', 'Foo', ('Err1', 'Err2'))
@@ -190,8 +190,6 @@ class BrowserWidget(Widget, BrowserView):
 
     """
 
-    implements(IBrowserWidget)
-
     _error = None
 
     def __init__(self, context, request):
@@ -208,6 +206,7 @@ class BrowserWidget(Widget, BrowserView):
         return ""
 
 
+@implementer(ISimpleInputWidget)
 class SimpleInputWidget(BrowserWidget, InputWidget):
     """A baseclass for simple HTML form widgets.
 
@@ -357,7 +356,7 @@ class SimpleInputWidget(BrowserWidget, InputWidget):
         >>> widget.getInputValue()
         32.0
         >>> widget()
-        u'<input class="textType" id="field.price" name="field.price" type="text" value="32.00"  />'
+        u'<input class="textType" id="field.price" name="field.price" required="True" type="text" value="32.00"  />'
 
         >>> request = TestRequest(form={'field.price': u'<p>foo</p>'})
         >>> widget = FloatWidget(field, request)
@@ -367,14 +366,12 @@ class SimpleInputWidget(BrowserWidget, InputWidget):
         ...     print error.doc()
         Invalid floating point data
         >>> widget()
-        u'<input class="textType" id="field.price" name="field.price" type="text" value="&lt;p&gt;foo&lt;/p&gt;"  />'
+        u'<input class="textType" id="field.price" name="field.price" required="True" type="text" value="&lt;p&gt;foo&lt;/p&gt;"  />'
 
 
     >>> tearDown()
     """
 
-    implements(ISimpleInputWidget)
-
     tag = u'input'
     type = u'text'
     cssClass = u''
@@ -524,6 +521,7 @@ class SimpleInputWidget(BrowserWidget, InputWidget):
                              id=self.name,
                              value=self._getFormValue(),
                              cssClass=self.cssClass,
+                             required=self.required,
                              extra=self.extra)
 
     def hidden(self):
